require_relative 'spec_helper'

describe Sir::Parser do
  let(:state) { Sir::Fixtures::LINE_WITH_VARIABLE[:state] }
  let(:path) { File.expand_path '../fixtures/commands.html', __FILE__ }
  subject    { Sir::Parser.new(path, state) }

  describe '.get_commands' do
    it 'Get as many commands as file have' do
      expect(subject.get_commands.size).
        to eq(Sir::Fixtures::COMMANDS_INFO[:commands_amount])
    end

    it 'Get first command properly' do
      expect(subject.get_commands.first).
        to eq(Sir::Fixtures::COMMANDS_INFO[:first_command])
    end
  end

  describe '.replace_variables' do
    it 'can replace varible like #{YO}' do
      input = Sir::Fixtures::LINE_WITH_VARIABLE[:in]
      out = Sir::Fixtures::LINE_WITH_VARIABLE[:out]
      expect(subject.send(:replace_variables, input)).to eq(out)
    end

    it 'can handle incorrect variables' do
      input = Sir::Fixtures::LINE_WITH_VARIABLE[:incorrect_in]
      out = Sir::Fixtures::LINE_WITH_VARIABLE[:incorrect_out]
      expect(subject.send(:replace_variables, input)).to eq(out)
    end
  end

  describe '.parse_equation' do
    it 'can split variables with equation' do
      input = Sir::Fixtures::LINE_WITH_EQUATION[:in]
      out = Sir::Fixtures::LINE_WITH_EQUATION[:out]
      expect(subject.send(:parse_equation, input)).to eq(out)
    end

    it 'can handle incorrect variables' do
      input = Sir::Fixtures::LINE_WITH_EQUATION[:incorrect_in]
      out = Sir::Fixtures::LINE_WITH_EQUATION[:incorrect_in]
      expect(subject.send(:parse_equation, input)).to eq(out)
    end

    it 'parse attributes' do
      input = Sir::Fixtures::LINE_WITH_EQUATION[:with_attr]
      out = Sir::Fixtures::LINE_WITH_EQUATION[:out_with_attr]
      expect(subject.send(:parse_equation, input)).to eq(out)
    end
  end

  describe '.parse_equation_with_regex' do
    it 'can split variables with equation' do
      input = Sir::Fixtures::LINE_WITH_REGEX[:in]
      out = Sir::Fixtures::LINE_WITH_REGEX[:out]
      expect(subject.send(:parse_equation_with_regex, input)).to eq(out)
    end

    it 'can handle incorrect variables' do
      input = Sir::Fixtures::LINE_WITH_REGEX[:incorrect_in]
      out = Sir::Fixtures::LINE_WITH_REGEX[:incorrect_in]
      expect(subject.send(:parse_equation_with_regex, input)).to eq(out)
    end
  end

  describe '.parse_command' do
    it 'parse "open ${BOX}/application"' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:open][:input])).
        to eq(Sir::Fixtures::COMMANDS[:open][:result])
    end

    it 'parse pause' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:pause][:input])).
        to eq(Sir::Fixtures::COMMANDS[:pause][:result])
    end

    it 'parse select' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:select][:input])).
        to eq(Sir::Fixtures::COMMANDS[:select][:result])
    end

    it 'parse click' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:click][:input])).
        to eq(Sir::Fixtures::COMMANDS[:click][:result])
    end

    it 'parse click_and_wait' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:click_and_wait][:input])).
        to eq(Sir::Fixtures::COMMANDS[:click_and_wait][:result])
    end

    it 'parse wait_for_element' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:wait_for_element][:input])).
        to eq(Sir::Fixtures::COMMANDS[:wait_for_element][:result])
    end

    it 'parse wait_for_loading' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:wait_for_loading][:input])).
        to eq(Sir::Fixtures::COMMANDS[:wait_for_loading][:result])
    end

    it 'parse wait_for_text' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:wait_for_text][:input])).
        to eq(Sir::Fixtures::COMMANDS[:wait_for_text][:result])
    end

    it 'parse state_set_url' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:state_set_url][:input])).
        to eq(Sir::Fixtures::COMMANDS[:state_set_url][:result])
    end

    it 'parse state_set_text' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:state_set_text][:input])).
        to eq(Sir::Fixtures::COMMANDS[:state_set_text][:result])
    end

    it 'parse expect_text' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:expect_text][:input])).
        to eq(Sir::Fixtures::COMMANDS[:expect_text][:result])
    end

    it 'parse find_and_set' do
      expect(subject.parse_command(*Sir::Fixtures::COMMANDS[:find_and_set][:input])).
        to eq(Sir::Fixtures::COMMANDS[:find_and_set][:result])
    end
  end

  context '#run' do
    it 'summarize script work' do
      path = File.expand_path 'fixtures/short_test.html', __dir__
      expect(Sir::Parser.run!(path)).to eq(Sir::Fixtures::SHORT_TEST_RESULT)
    end

  end
end
